/* This file is an image processing operation for GEGL
 *
 * GEGL is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * GEGL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
 *
 * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
 * Add your date here - Plugin Name and username/real name here.
 */

#include "config.h"
#include <glib/gi18n-lib.h>

#ifdef GEGL_PROPERTIES

/*
Search through GEGL's source code in /operations/common to find templates.

Example Templates below that call gaussian blur and hsl color overlay together'
 */


enum_start (gegl_emboss_type2)
  enum_value (GEGL_EMBOSS_TYPE_EMBOSS2,  "emboss",  N_("Emboss"))
  enum_value (GEGL_EMBOSS_TYPE_BUMPMAP2, "bumpmap", N_("Bumpmap (preserve original colors)"))
enum_end (GeglEmbossType2)

property_enum (type, _("Emboss Type"),
               GeglEmbossType2, gegl_emboss_type2, GEGL_EMBOSS_TYPE_EMBOSS2)
    description(_("Rendering type"))



property_double (std_dev, _("Blur Slider"), 0.0)
   description (_("Add a Gaussian Blur"))
   value_range (0.0, 20.0)
   ui_range    (0.24, 20.0)
   ui_gamma    (3.0)
   ui_meta     ("unit", "pixel-distance")



property_color (value, _("Color"), "#00b7ff")
    description (_("The color to paint over the input"))


property_double (opacity, _("Opacity of HSL Color"), 1.0)
    description (_("Global opacity value that is always used on top of the optional auxiliary input buffer."))
    value_range (0.0, 1.0)
    ui_range    (0.0, 1.0)

#else

#define GEGL_OP_META
#define GEGL_OP_NAME     my_new_gegl_plugin
#define GEGL_OP_C_SOURCE my_new_gegl_plugin.c

#include "gegl-op.h"

static void attach (GeglOperation *operation)
{
  GeglNode *gegl = operation->node;
  GeglNode *input, *output, *emboss, *gaussian, *hslcolor,  *opacity, *coloroverlay;

  input    = gegl_node_get_input_proxy (gegl, "input");
  output   = gegl_node_get_output_proxy (gegl, "output");

  coloroverlay = gegl_node_new_child (gegl,
                                  "operation", "gegl:color-overlay",
                                  NULL);

  emboss = gegl_node_new_child (gegl,
                                  "operation", "gegl:emboss",
                                  NULL);


  gaussian    = gegl_node_new_child (gegl,
                                  "operation", "gegl:gaussian-blur",
                                  NULL);

hslcolor = gegl_node_new_child (gegl,
                                  "operation", "gimp:layer-mode", "layer-mode", 39, NULL);

  opacity    = gegl_node_new_child (gegl,
                                  "operation", "gegl:opacity",
                                  NULL);



  gegl_node_link_many (input, hslcolor, gaussian, output, NULL);
  gegl_node_link_many (input, coloroverlay, opacity, NULL);

  gegl_node_connect_from (hslcolor, "aux", opacity, "output");

  gegl_operation_meta_redirect (operation, "std_dev", gaussian, "std-dev-x");
  gegl_operation_meta_redirect (operation, "std_dev", gaussian, "std-dev-y");
  gegl_operation_meta_redirect (operation, "value", coloroverlay, "value");
  gegl_operation_meta_redirect (operation, "opacity", opacity, "value");
  gegl_operation_meta_redirect (operation, "type", emboss, "type");


}

static void
gegl_op_class_init (GeglOpClass *klass)
{
  GeglOperationClass *operation_class;

  operation_class = GEGL_OPERATION_CLASS (klass);

  operation_class->attach = attach;

  gegl_operation_class_set_keys (operation_class,
    "name",        "gegl:newplugin",
    "title",       _("My New GEGL Plugin"),
    "categories",  "newplugin",
    "reference-hash", "newplugin6a238a5125700fc254001b2ac",
    "description", _("Template for GEGL Plugins"),
    NULL);
}

#endif
